from datetime import date

from namelist_reader import read_namelist

# Constants
LOG_FILE_NAME = 'amip.log'
# Reference years of the dataset
DATA_YREF_MIN = 1870
DATA_YREF_MAX = 2016

def days_since_refdate(yy, mm, dd):
    d0 = date(DATA_YREF_MIN, 1, 1)
    d1 = date(yy, mm, dd)
    delta = d1 - d0
    return delta.days


def is_leap(year):
    return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)


class AMIPForcing:
    def __init__(self):
        # Open log_file
        self.log_file = open(LOG_FILE_NAME, 'w')

        # Read RunLengthSec,TimeStepSec from namelist
        self.RunLengthSec, self.TimeStepSec, self.StartYear, self.StartMonth, \
            self.StartDay, self.FixYear, self.FileListSST, self.FileListSIC, self.LDebug, \
            self.LInterpolate = read_namelist()

        # valid FixYear to get a seasonal cycle, or set one if none and running outside AMIP window
        if (self.FixYear > 0):
            self.FixYear = min(DATA_YREF_MAX, max(DATA_YREF_MIN, self.FixYear))
        else:
            if ((self.StartYear < DATA_YREF_MIN) or (self.StartYear > DATA_YREF_MAX)):
                # set pseudo fixyear,
                self.FixYear = min(DATA_YREF_MAX, max(DATA_YREF_MIN, self.StartYear))
                # but do not overshoot
                if ((self.StartYear < DATA_YREF_MIN) and (self.RunLengthSec > (DATA_YREF_MIN-self.StartYear+1)*3600*24*365)):
                    raise Exception("RunLengthSec is too long for StartYear less than DATA_YREF_MIN")

        self.lstored = False
        self.lleapfx1 = False
        self.lleapfx2 = False


    # Init method
    def setup_amip_forcing(self):
        yy = self.StartYear
        mm = self.StartMonth
        dd = self.StartDay
        #  time axis unit = days
        deltaT = self.TimeStepSec/86400.
        # simulated year
        yr_sim = yy
        # amip_reader year
        yyref = self.FixYear if self.FixYear > 0 else yy

        t = days_since_refdate(yyref, mm, dd)
        t0101 = days_since_refdate(yyref, 1, 1)
        t1231 = days_since_refdate(yyref, 12, 31)

        # find out when we switch year in amip_year according to simulated year
        tnewyear = t0101 + 365
        if (is_leap(yy)):
            tnewyear = t0101 + 366

        if (self.LDebug):
            print('AMIP INIT - simulated/amip/fixed years:', yy, yyref, self.FixYear, file=self.log_file)
            print(f'("AMIP INIT - tstart  :", {t})', file=self.log_file)
            print(f'("AMIP INIT - t0101   :", {t0101})', file=self.log_file)
            print(f'("AMIP INIT - t1231   :", {t1231})', file=self.log_file)
            print(f'("AMIP INIT - tnewyear:", {tnewyear})', file=self.log_file)

        # initialise with dummy values
        t1=-9999.
        t2=-9999.

        # point to first file
        ifile = 1
        lfirst = True

        # setup BC for FixYear
        if self.FixYear>0:
            if not self.lstored:
               self.setup_fixyear_bc()
               self.lstored = True
            self.discordant_leap_years(yr_sim, self.FixYear)

    def discordant_leap_years(simyr, fixyr):
        pass

    def finalise_amip_forcing(self):
        self.log_file.close()
        # TODO
